import os

def account_file(account_address=None, testnet=False):
    if account_address == None:
        if testnet == False:
            with open(os.path.expanduser("~/.ether/mainnet_account.csv"), "r") as file:
                account = file.read()
            return account
        if testnet == True:
            with open(os.path.expanduser("~/.ether/testnet_account.csv"), "r") as file:
                test_account = file.read()
            return test_account
    if account_address != None:
        if testnet == False:
            try:
                with open(os.path.expanduser("~/.ether/mainnet_account.csv"), "w") as file:
                    file.write(account_address)
            except:
                os.mkdir(os.path.expanduser("~/.ether"))
                with open(os.path.expanduser("~/.ether/mainnet_account.csv"), "w") as file:
                    file.write(account_address)
        if testnet == True:
            try:
                with open(os.path.expanduser("~/.ether/testnet_account.csv"), "w") as file:
                    file.write(account_address)
            except:
                os.mkdir(os.path.expanduser("~/.ether"))
                with open(os.path.expanduser("~/.ether/testnet_account.csv"), "w") as file:
                    file.write(account_address)

def init_account(testnet=False):
    account_address = input("ACCOUNT: ")
    if testnet == False:
        account_file(account_address)
        print("Watch only account has been set.")
    if testnet == True:
        account_file(account_address)
        print("Watch only testnet account has been set.")

def provider_file(provider=None, chain_id=None, testnet=False):
    if provider == None:
        if testnet == False:
            with open(os.path.expanduser("~/.ether/mainnet_provider.csv"), "r") as file:
                mainnet = file.read()
            return mainnet
        if testnet == True:
            with open(os.path.expanduser("~/.ether/testnet_provider.csv"), "r") as file:
                test_network = file.read()
            return test_network
    if provider != None:
        if testnet == False:
            try:
                with open(os.path.expanduser("~/.ether/mainnet_provider.csv"), "w") as file:
                    file.write(provider)
            except:
                os.mkdir(os.path.expanduser("~/.ether"))
                with open(os.path.expanduser("~/.ether/mainnet_provider.csv"), "w") as file:
                    file.write(provider)
        if testnet == True:
            try:
                with open(os.path.expanduser("~/.ether/testnet_provider.csv"), "w") as file:
                    file.write(provider)
            except:
                os.mkdir(os.path.expanduser("~/.ether"))
                with open(os.path.expanduser("~/.ether/testnet_provider.csv"), "w") as file:
                    file.write(provider)
            with open(os.path.expanduser("~/.ether/testnet_chain"), "w") as file:
                file.write(chain_id)

def testnet_chain_id_file():
    with open(os.path.expanduser("~/.ether/testnet_chain_id.csv"), "r") as file:
        chain_id = file.read()
    return chain_id

def wallet_path(path=None):
    if path == None:
        with open(os.path.expanduser("~/.ether/path.csv"), "r") as file:
            location = file.read()
        return location
    if path != None:
        with open(os.path.expanduser("~/.ether/path.csv"), "w") as file:
            file.write(path)



from requests import get

def eth_price(multiplier):
    ethereum = get("https://min-api.cryptocompare.com/data/price?fsym=ETH&tsyms=USD").json()
    usd_eth = ethereum["USD"]
    multiplied_eth = usd_eth * float(multiplier)
    return multiplied_eth

def eth_estimate(total, unit=1):
    gwei_gas = int(total) * int(unit)
    eth_gas = gwei_gas * 0.000000001
    cost = eth_price(1) * eth_gas
    return cost

def unit_convert(type=None, amount=None):
    if type == "eth":
        eth = 1 * float(amount)
        gwei = 1000000000 * float(amount)
        wei = 1000000000000000000 * float(amount)
        return eth, gwei, wei
    elif type == "gwei":
        eth = 0.000000001 * float(amount)
        gwei = 1 * float(amount)
        wei = 1000000000 * float(amount)
        return eth, gwei, wei
    elif type == "wei":
        eth = 0.000000000000000001 * float(amount)
        gwei = 0.000000001 * float(amount)
        wei = 1 * float(amount)
        return eth, gwei, wei



from ether import transaction, local, utils
from cryptography.fernet import Fernet
from mnemonic import Mnemonic
from hdwallet import HDWallet
from hdwallet.symbols import ETH
from getpass import getpass
import os

def seed_phrase():
    lang = Mnemonic("english")
    mnemonic = lang.generate(strength=256)
    return lang, mnemonic

def try_path(path):
    mnemonic = seed_phrase()[1]
    try:
        create_file_pair(path, mnemonic)
    except:
        print("Error has occured.")

def create_file_pair(path, mnemonic):
    key_file(path)
    mnemonic_file(path, mnemonic)
    try:
        with open(os.path.expanduser("~/.wallet/path.csv"), "w") as file:
            file.write(path.strip())
    except:
        os.mkdir(os.path.expanduser("~/.wallet"))
        with open(os.path.expanduser("~/.wallet/path.csv"), "w") as file:
            file.write(path.strip())
    print(f"\nSeed phrase and encryption key have been generated in the external drive\n\n{mnemonic}\n\nWrite backup mnemonic and store drive in secure location")

def key_file(path):
    Key = Fernet.generate_key()
    with open(f"{path}/encryption.key", "wb") as key:
        key.write(Key)

def mnemonic_file(path, mnemonic):
    with open(f"{path}/encryption.key", "rb") as key:
        Key = key.read()
    fernet = Fernet(Key)
    encrypted_mnemonic = fernet.encrypt(mnemonic.encode("utf-8"))
    with open(f"{path}/key.csv", "wb") as file:
        file.write(encrypted_mnemonic)

def decrypt_mnemonic(path):
    with open(f"{path}/encryption.key", "rb") as key:
        Key = key.read()
    fernet = Fernet(Key)
    with open(f"{path}/key.csv", "rb") as file:
        encrypted_mnemonic = file.read()
    decrypted_mnemonic = fernet.decrypt(encrypted_mnemonic)
    return decrypted_mnemonic

def derive_wallet(mnemonic, index, passphrase=None, private=False):
    wallet = HDWallet(ETH)
    lang = seed_phrase()[0]
    if passphrase != None:
        seed = lang.to_seed(mnemonic, passphrase=passphrase)
    else:
        seed = lang.to_seed(mnemonic)
    wallet.from_seed(seed=seed.hex())
    wallet.from_path(path=f"m/44'/60'/0'/0/{index}")
    if private == True:
        private_key = wallet.private_key()
        account_address = wallet.p2pkh_address()
        return private_key, account_address
    if private == False:
        account_address = wallet.p2pkh_address()
        return account_address

def derivation(path, index, passphrase=None, private=False, mnemonic=None, printer=False):
    if passphrase == "":
        passphrase = None
    if mnemonic != None:
        wallet = HDWallet(ETH)
        check_mnemonic = wallet.from_mnemonic(mnemonic=mnemonic)
        check_mnemonic.mnemonic()
        if private == True:
            key_pair = derive_wallet(mnemonic, index, passphrase, private)
            if printer == True:
                print(f"ACCOUNT: {key_pair[1]}")
                print(f"PRIVATE: {key_pair[0]}")
            return key_pair[0], key_pair[1]
        if private == False:
            account_address = derive_wallet(mnemonic, index, passphrase, private)
            if printer == True:
                print(f"ACCOUNT: {account_address}")
            return account_address
    else:
        if private == True:
            decrypted_mnemonic = decrypt_mnemonic(path.strip())
            key_pair = derive_wallet(decrypted_mnemonic, index, passphrase, private)
            if printer == True:
                print(f"ACCOUNT: {key_pair[1]}")
                print(f"PRIVATE: {key_pair[0]}")
            return key_pair[0], key_pair[1]
        if private == False:
            decrypted_mnemonic = decrypt_mnemonic(path.strip())
            account_address = derive_wallet(decrypted_mnemonic, index, passphrase, private)
            if printer == True:
                print(f"ACCOUNT: {account_address}")
            return account_address

def account(private=False):
    try:
        seed_phrase = None
        ext_file_path = local.wallet_path()
        derivation(ext_file_path, "0")
        mnemonic = False
    except:
        print("Use command `ether init location` to set wallet path.")
        seed_phrase = getpass("MNEMONIC: ")
        ext_file_path = None
        mnemonic = True
    passphrase = getpass("PASSPHRASE: ")
    account_number = input("INDEX: ")
    if account_number == "":
        account_number = "0"
    if mnemonic == True:
        if private == True:
            derivation(ext_file_path, account_number, passphrase=passphrase, private=True, mnemonic=seed_phrase, printer=True)
        if private == False:
            derivation(ext_file_path, account_number, passphrase=passphrase, mnemonic=seed_phrase, printer=True)
    if mnemonic == False:
        if private == True:
            derivation(ext_file_path, account_number, passphrase=passphrase, private=True, printer=True)
        if private == False:
            derivation(ext_file_path, account_number, passphrase=passphrase, printer=True)

def balance(testnet=False):
    try:
        status = True
        if testnet == False:
            account = local.account_file()
            try:
                provider_data = local.provider_file()
            except:
                status = False
                if testnet == False:
                    print("Use command `ether init provider` to set provider.")
                if testnet == True:
                    print("Use command `ether init provider --testnet` to set provider.")
        if testnet == True:
            account = local.account_file(testnet=True)
            try:
                provider_data = local.provider_file(testnet=True)
            except:
                status = False
                if testnet == False:
                    print("Use command `ether init provider` to set provider.")
                if testnet == True:
                    print("Use command `ether init provider --testnet` to set provider.")
        if status == True:
            provider_info = transaction.provider(provider_data)
            balance = transaction.get_balance(provider_info, account)
            print(f"{account}\n{balance} ETH")
            if testnet == False:
                try:
                    print(f"${utils.eth_price(balance):.2f}")
                except:
                    pass
    except:
        if testnet == False:
            print("Use command `ether init account` to set watch only account.")
        if testnet == True:
            print("Use command `ether init account --testnet` to set watch only account.")



from ether import wallet, transaction, local, utils
from sys import argv
from getpass import getpass

msg_menu = """▄▄▄ .▄▄▄▄▄ ▄ .▄▄▄▄ .▄▄▄  
▀▄.▀·•██  ██▪▐█▀▄.▀·▀▄ █·
▐▀▀▪▄ ▐█.▪██▀▐█▐▀▀▪▄▐▀▀▄ 
▐█▄▄▌ ▐█▌·██▌▐▀▐█▄▄▌▐█•█▌
 ▀▀▀  ▀▀▀ ▀▀▀ · ▀▀▀ .▀  ▀
v0.1.4"""

msg_commands = """ether account
ether account --testnet
ether convert eth <amount>
ether convert gwei <amount>
ether convert wei <amount>
ether keys
ether price
ether price <amount>
ether gas <total>
ether gas <total> <price>
ether init
ether init account
ether init account --testnet
ether init location
ether init provider
ether init provider --testnet
ether mnemonic
ether send
ether send --testnet"""

def main():
    args = argv[1:]
    if len(args) == 0:
        print(msg_menu)
    try:
        if len(args) == 1:
            arg2 = argv[1]
            if arg2 == "init":
                ext_file_path = input("DRAG AND DROP EXTERNAL DRIVE: ")
                wallet.try_path(path=ext_file_path.strip())
            if arg2 == "account":
                wallet.balance()
            if arg2 == "keys":
                wallet.account(private=True)
            if arg2 == "mnemonic":
                print(f"\n{wallet.mnemonic()}\n")
            if arg2 == "price":
                ether = utils.eth_price(1)
                print(f"${ether:.2f}")
            if arg2 == "commands":
                print(msg_commands)
            if arg2 == "send":
                transaction.send_transaction()
        elif len(args) == 2:
            arg2 = argv[1]
            arg3 = argv[2]
            if arg2 == "init" and arg3 == "account":
                local.init_account()
            elif arg2 == "init" and arg3 == "provider":
                provider = getpass("PROVIDER: ")
                local.provider_file(provider=provider, chain_id=1)
                print("Mainnet provider has been set.")
            elif arg2 == "init" and arg3 == "location":
                ext_file_path = input("DRAG AND DROP EXTERNAL DRIVE: ")
                local.wallet_path(path=ext_file_path)
                print("Wallet location has been set.")
            if arg2 == "account" and arg3 == "--testnet":
                wallet.balance(testnet=True)
            if arg2 == "price":
                ether = utils.eth_price(arg3)
                print(f"${ether:.2f}")
            if arg2 == "gas":
                estimate = utils.eth_estimate(arg3, 1)
                print(f"${estimate:.2f}")
            if arg2 == "send" and arg3 == "--testnet":
                transaction.send_transaction(testnet=True)
        elif len(args) == 3:
            arg2 = argv[1]
            arg3 = argv[2]
            arg4 = argv[3]
            if arg2 == "init" and arg3 == "provider" and arg4 == "--testnet":
                provider = getpass("PROVIDER: ")
                chain_id = input("CHAIN ID: ")
                local.provider_file(provider=provider, chain_id=chain_id, testnet=True)
                print("Testnet provider has been set.")
            elif arg2 == "init" and arg3 == "account" and arg4 == "--testnet":
                local.init_account(testnet=True)
            if arg2 == "gas":
                estimate = utils.eth_estimate(arg3, arg4)
                print(f"${estimate:.2f}")
            if arg2 == "convert":
                convert = utils.unit_convert(arg3, arg4)
                print(f"{convert[0]:.16f}".rstrip("0").rstrip("."), "ETH")
                print(f"{convert[1]:.16f}".rstrip("0").rstrip("."), "GWEI")
                print(f"{convert[2]:.16f}".rstrip("0").rstrip("."), "WEI")
    except Exception as e:
        print(e)

if __name__ == "__main__":
    main()



from ether import local, wallet
from web3 import Web3, HTTPProvider
from web3.middleware import geth_poa_middleware
from getpass import getpass

def provider(network):
    provider = Web3(HTTPProvider(network))
    provider.middleware_onion.inject(geth_poa_middleware, layer=0)
    return provider

def get_balance(provider=None, account_address=None):
    return provider.eth.get_balance(str(account_address)) / 1000000000000000000

def core_transaction(provider=None, to=None, amount=None, gas_price=None, data=None, key_pair=None, chain_id=1):
    if chain_id == 1:
        chain = 1
    elif chain_id == "ropsten":
        chain = 3
    elif chain_id == "kovan":
        chain = 42
    elif chain_id == "rinkeby":
        chain = 4
    else:
        chain = chain_id
    if data == None:
        txn_data = b""
    else:
        note = data.encode()
    signed_tx = provider.eth.account.signTransaction({
        "maxFeePerGas" : gas_price * 1000000000,
        "maxPriorityFeePerGas" : 2000000000,
        "to" : to,
        "from" : key_pair[1],
        "value" : int(amount * 1000000000000000000),
        "nonce" : provider.eth.get_transaction_count(key_pair[1]),
        "gas" : 21000,
        "data" : txn_data,
        "chainId" : chain,
        },
        key_pair[0]
    )
    tx_hash = provider.eth.send_raw_transaction(signed_tx.rawTransaction)
    print("Processing transaction...")
    tx_receipt = provider.eth.wait_for_transaction_receipt(tx_hash)
    print("Transaction completed.")
    return tx_receipt.transactionHash.hex()

def send_transaction(data=None, testnet=False):
    try:
        seed_phrase = None
        ext_file_path = local.wallet_path()
        wallet.derivation(ext_file_path, "0")
    except:
        print("Use command `ether init location` to set wallet path.")
        seed_phrase = getpass("MNEMONIC: ")
        ext_file_path = None
    passphrase = getpass("PASSPHRASE: ")
    account_number = input("INDEX: ")
    if account_number == "":
        account_number = "0"
    if passphrase != "":
        key_pair = wallet.derivation(ext_file_path, account_number, passphrase=passphrase, private=True, mnemonic=seed_phrase)
    else:
        key_pair = wallet.derivation(ext_file_path, account_number, private=True, mnemonic=seed_phrase)
    provider_info = local.provider_file(testnet=testnet)
    provider_data = provider(provider_info.strip())
    balance = get_balance(provider_data, key_pair[1])
    print(f"ACCOUNT: {key_pair[1]}\nBALANCE: {balance} ETH")
    menu = input("CONTINUE? Y/N ")
    if menu.lower() == "y":
        to = input("TO: ")
        amount = input("ETH AMOUNT: ")
        gas = input("GAS PRICE: ")
        if testnet == True:
            txn = core_transaction(provider=provider_data, to=to, amount=float(amount), gas_price=int(gas), data=data, key_pair=key_pair, chain_id=local.testnet_chain_id_file())
        if testnet == False:
            txn = core_transaction(provider=provider_data, to=to, amount=float(amount), gas_price=int(gas), data=data, key_pair=key_pair)
        print(txn)


from setuptools import setup
import pathlib

file_path = pathlib.Path(__file__).parent.resolve()
readme = (file_path/"README.md").read_text()

setup(
  name="ether-cli",
  version="0.1.5",
  description="Minimalistic wallet CLI for Ethereum",
  long_description=readme,
  long_description_content_type="text/markdown",
  url="https://github.com/0xver/ether",
  author="Sam Larsen",
  license="MIT",
  packages=["ether"],
  entry_points = {
    "console_scripts": ["ether=ether.cli:main"],
  },
  install_requires=[
      "cryptography", "mnemonic", "hdwallet", "web3", "requests",
  ],
  python_requires=">=3.9"
)
